package com.ahai.vgk_game_services

import android.app.Activity.RESULT_OK
import android.app.Activity
import android.content.Context
import android.content.Intent
import com.google.android.gms.auth.api.Auth
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInAccount
import com.google.android.gms.auth.api.signin.GoogleSignInClient
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import com.google.android.gms.auth.api.signin.GoogleSignInResult
import com.google.android.gms.drive.Drive
import com.google.android.gms.games.Games
import com.google.android.gms.games.SnapshotsClient
import com.google.android.gms.games.snapshot.Snapshot
import com.google.android.gms.games.snapshot.SnapshotMetadataChange
import java.io.IOException
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry

class VgkGameServicesPlugin : FlutterPlugin, MethodCallHandler, ActivityAware, PluginRegistry.ActivityResultListener {
    private var channel: MethodChannel? = null
    private var context: Context? = null
    private var activityPluginBinding: ActivityPluginBinding? = null
    private var activity: Activity? = null
    private var lastResult: Result? = null
    @Override
    fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.getBinaryMessenger(), "vgk_game_services")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.getApplicationContext()
    }
   
    private fun parseBooleanValue(`val`: Object?, defaultValue: Boolean): Boolean {
        return if (`val` == null || `val`.toString().equals("")) {
            defaultValue
        } else Boolean.parseBoolean(`val`.toString())
    }

    @Override
    fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        val fr: Result = result
        if (call.method.equals("signIn")) {
            val builder: GoogleSignInOptions.Builder = Builder(GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN).requestEmail()
            if (parseBooleanValue(call.argument("scopeSnapShot"), false)) {
                builder.requestScopes(Games.SCOPE_GAMES_SNAPSHOTS, Drive.SCOPE_APPFOLDER)
            }
            val signInOptions: GoogleSignInOptions = builder.build()
            val signInClient: GoogleSignInClient = GoogleSignIn.getClient(context, signInOptions)
            signInClient
            .silentSignIn()
            .addOnCompleteListener { task ->
                if (task.isSuccessful()) {
                    fr.success(PluginResult().setData(task.getResult().getEmail()).toMap())
                } else {
                    lastResult = fr
                    val intent: Intent = signInClient.getSignInIntent()
                    activity.startActivityForResult(intent, RC_SIGNIN)
                }
            }
        } else if (call.method.equals("saveSnapShot")) {
            val snapshotsClient: SnapshotsClient = Games.getSnapshotsClient(context, GoogleSignIn.getLastSignedInAccount(context))
            snapshotsClient.open(call.argument("name"), true, SnapshotsClient.RESOLUTION_POLICY_MOST_RECENTLY_MODIFIED)
                    .addOnCompleteListener { task ->
                        if (task.isSuccessful()) {
                            val snapshot: Snapshot = task.getResult().getData()
                            snapshot.getSnapshotContents().writeBytes(call.argument("data"))
                            val metadataChange: SnapshotMetadataChange = Builder()
                                    .setDescription(call.argument("description"))
                                    .build()
                            snapshotsClient.commitAndClose(snapshot, metadataChange)
                                    .addOnCompleteListener { task1 ->
                                        if (task1.isSuccessful()) {
                                            fr.success(PluginResult().toMap())
                                        } else {
                                            fr.success(PluginResult(task.getException().toString()).toMap())
                                        }
                                    }
                        } else {
                            fr.success(PluginResult(task.getException().toString()).toMap())
                        }
                    }
        } else if (call.method.equals("loadSnapShot")) {
            val snapshotsClient: SnapshotsClient = Games.getSnapshotsClient(context, GoogleSignIn.getLastSignedInAccount(context))
            snapshotsClient.open(call.argument("name"), false, SnapshotsClient.RESOLUTION_POLICY_MOST_RECENTLY_MODIFIED)
                    .addOnCompleteListener { task ->
                        if (task.isSuccessful()) {
                            val snapshot: Snapshot = task.getResult().getData()
                            try {
                                fr.success(PluginResult().setData(snapshot.getSnapshotContents().readFully()).toMap())
                            } catch (e: IOException) {
                                fr.success(PluginResult(e.toString()).toMap())
                            }
                        } else {
                            fr.success(PluginResult(task.getException().toString()).toMap())
                        }
                    }
        } else if (call.method.equals("increment")) {
            Games.getAchievementsClient(context, GoogleSignIn.getLastSignedInAccount(context))
                    .incrementImmediate(call.argument("id"), 1)
                    .addOnSuccessListener { task -> fr.success(PluginResult().toMap()) }.addOnFailureListener { e -> fr.success(PluginResult(e.toString()).toMap()) }
        } else if (call.method.equals("submitScore")) {
            val score: Int = call.argument("score")
            Games.getLeaderboardsClient(context, GoogleSignIn.getLastSignedInAccount(context))
                    .submitScoreImmediate(call.argument("id"), score)
                    .addOnSuccessListener { task -> fr.success(PluginResult().toMap()) }.addOnFailureListener { e -> fr.success(PluginResult(e.toString()).toMap()) }
        } else if (call.method.equals("showAchievements")) {
            Games.getAchievementsClient(context, GoogleSignIn.getLastSignedInAccount(context))
                    .getAchievementsIntent().addOnSuccessListener { intent ->
                        activity.startActivityForResult(intent, 0)
                        fr.success(PluginResult().toMap())
                    }.addOnFailureListener { e -> fr.success(PluginResult(e.toString()).toMap()) }
        } else if (call.method.equals("showLeaderboards")) {
            Games.getLeaderboardsClient(context, GoogleSignIn.getLastSignedInAccount(context))
                    .getAllLeaderboardsIntent().addOnSuccessListener { intent ->
                        activity.startActivityForResult(intent, 0)
                        fr.success(PluginResult().toMap())
                    }.addOnFailureListener { e -> fr.success(PluginResult(e.toString()).toMap()) }
        } else {
            result.notImplemented()
        }
    }

    @Override
    fun onDetachedFromEngine(@NonNull binding: FlutterPluginBinding?) {
        channel.setMethodCallHandler(null)
    }

    @Override
    fun onAttachedToActivity(@NonNull activityPluginBinding: ActivityPluginBinding) {
        this.activityPluginBinding = activityPluginBinding
        activity = activityPluginBinding.getActivity()
        activityPluginBinding.addActivityResultListener(this)
    }

    @Override
    fun onDetachedFromActivityForConfigChanges() {
        onDetachedFromActivity()
    }

    @Override
    fun onReattachedToActivityForConfigChanges(@NonNull activityPluginBinding: ActivityPluginBinding) {
        onAttachedToActivity(activityPluginBinding)
    }

    @Override
    fun onDetachedFromActivity() {
        if (activityPluginBinding != null) {
            activityPluginBinding.removeActivityResultListener(this)
            activityPluginBinding = null
            activity = null
        }
    }

    @Override
    fun onActivityResult(requestCode: Int, resultCode: Int, intent: Intent?): Boolean {
        if (requestCode == RC_SIGNIN && lastResult != null) {
            if (resultCode == RESULT_OK && intent != null) {
                val result: GoogleSignInResult = Auth.GoogleSignInApi.getSignInResultFromIntent(intent)
                if (result.isSuccess()) {
                    val signedInAccount: GoogleSignInAccount = result.getSignInAccount()
                    assert(signedInAccount != null)
                    lastResult.success(PluginResult().setData(signedInAccount.getEmail()).toMap())
                } else {
                    lastResult.success(PluginResult("Login failed").toMap())
                }
            } else {
                lastResult.success(PluginResult("User canceled").toMap())
            }
            lastResult = null
            return true
        }
        return false
    }

    companion object {
        private const val RC_SIGNIN = 9001
        private const val RC_SNAPSHOT = 9002
    }
}