import 'dart:async';
import 'dart:typed_data';
import 'package:flutter/services.dart';

class PluginResult {
  bool success = false;
  String? message;

  PluginResult(Map<dynamic, dynamic> map) {
    this.success = map["success"];
    this.message = map["exception"];
  }
}

class SignInResult extends PluginResult {
  String? email;

  SignInResult(Map<dynamic, dynamic> map) : super(map) {
    if (this.success) {
      this.email = map["data"];
    }
  }
}

/// the result of LoadSnapshot
class LoadSnapshotResult extends PluginResult {
  Uint8List? data;

  LoadSnapshotResult(Map<dynamic, dynamic> map) : super(map) {
    if (this.success) {
      this.data = map["data"];
    }
  }
}

class VgkGameServices {
  static const MethodChannel _channel =
      const MethodChannel('vgk_game_services');

  static Future<SignInResult> signIn({bool scopeSnapShot = false}) async {
    try {
      Map<dynamic, dynamic> result = await _channel
          .invokeMethod('signIn', {"scopeSnapShot": scopeSnapShot});
      return SignInResult(result);
    } catch (e) {
      return SignInResult({"success": false, "exception": e.toString()});
    }
  }

  static Future<LoadSnapshotResult> loadSnapShot(String name) async {
    try {
      Map<dynamic, dynamic> map =
          await _channel.invokeMethod('loadSnapShot', {"name": name});
      return LoadSnapshotResult(map);
    } catch (e) {
      return LoadSnapshotResult({"success": false, "exception": e.toString()});
    }
  }

  static Future<PluginResult> saveSnapShot(
      String name, Uint8List data, String description) async {
    try {
      Map<dynamic, dynamic> result = await _channel.invokeMethod('saveSnapShot',
          {"name": name, "data": data, "description": description});
      return PluginResult(result);
    } catch (e) {
      return PluginResult({"success": false, "exception": e.toString()});
    }
  }

  static Future<PluginResult> submitScore(
      String leaderBoardId, int score) async {
    try {
      Map<dynamic, dynamic> result = await _channel
          .invokeMethod('submitScore', {"id": leaderBoardId, "score": score});
      return PluginResult(result);
    } catch (e) {
      return PluginResult({"success": false, "exception": e.toString()});
    }
  }

  static Future<PluginResult> increment(String achievementId) async {
    try {
      Map<dynamic, dynamic> result =
          await _channel.invokeMethod('increment', {"id": achievementId});
      return PluginResult(result);
    } catch (e) {
      return PluginResult({"success": false, "exception": e.toString()});
    }
  }

  static void showLeaderboards() async {
    try {
      await _channel.invokeMethod('showLeaderboards');
    } catch (e) {}
  }

  static void showAchievements() async {
    try {
      await _channel.invokeMethod('showAchievements');
    } catch (e) {}
  }
}
